package com.ruoyi.bst.vipOrder.service;

import java.util.List;

import com.ruoyi.bst.pay.domain.vo.DoPayVO;
import com.ruoyi.bst.vipOrder.domain.VipOrder;
import com.ruoyi.bst.vipOrder.domain.VipOrderQuery;
import com.ruoyi.bst.vipOrder.domain.VipOrderVO;
import com.ruoyi.bst.vipOrder.domain.dto.VipOrderCreateDTO;

/**
 * 卡券订单Service接口
 *
 * @author ruoyi
 * @date 2025-05-24
 */
public interface VipOrderService
{
    /**
     * 查询卡券订单
     *
     * @param id 卡券订单主键
     * @return 卡券订单
     */
    public VipOrderVO selectVipOrderById(Long id, boolean scope);

    default VipOrderVO selectVipOrderById(Long id) {
        return selectVipOrderById(id, false);
    }

    /**
     * 查询卡券订单列表
     *
     * @param vipOrder 卡券订单
     * @return 卡券订单集合
     */
    public List<VipOrderVO> selectVipOrderList(VipOrderQuery vipOrder);

    /**
     * 新增卡券订单
     *
     * @param vipOrder 卡券订单
     * @return 结果
     */
    public int insertVipOrder(VipOrder vipOrder);

    /**
     * 修改卡券订单
     *
     * @param vipOrder 卡券订单
     * @return 结果
     */
    public int updateVipOrder(VipOrder vipOrder);

    /**
     * 批量删除卡券订单
     *
     * @param ids 需要删除的卡券订单主键集合
     * @return 结果
     */
    public int deleteVipOrderByIds(Long[] ids);

    /**
     * 删除卡券订单信息
     *
     * @param id 卡券订单主键
     * @return 结果
     */
    public int deleteVipOrderById(Long id);

    /**
     * 创建并支付卡券订单
     *
     * @param dto 卡券订单创建DTO
     * @return 支付结果
     */
    public DoPayVO createOrder(VipOrderCreateDTO dto);

    /**
     * 查询卡券订单详情
     *
     * @param id 卡券订单主键
     * @param userId 用户ID
     * @return 卡券订单详情
     */
    public VipOrderVO selectDetailByUserId(Long id, Long userId);

    /**
     * 根据查询条件更新卡券订单
     *
     * @param data 卡券订单
     * @param query 查询条件
     * @return 结果
     */
    public int updateByQuery(VipOrder data, VipOrderQuery query);
}
